@@ -50,28 +50,38 @@
  * your accounts are saved so that the next time you start Purple,
  * your accounts will be set to their last known statuses.  There
  * is also a list of saved statuses that are written to the
- * status.xml file.  Also, each PurpleStatus has a "savable" boolean.
- * If "savable" is set to FALSE then the status is NEVER saved.
+ * status.xml file.  Also, each PurpleStatus has a "saveable" boolean.
+ * If "saveable" is set to FALSE then the status is NEVER saved.
  * All PurpleStatuses should be inside a PurplePresence.
  *
  *
- * A PurpleStatus is either "indepedent" or "exclusive."
- * Independent statuses can be active or inactive and it doesn't
+ * A PurpleStatus is either "independent" or "exclusive."
+ * Independent statuses can be active or inactive and they don't
  * affect anything else.  However, you can only have one exclusive
- * status per PurplePresence.  If you activate one exlusive status,
+ * status per PurplePresence.  If you activate one exclusive status,
  * then the previous exclusive status is automatically deactivated.
  *
  * A PurplePresence is like a collection of PurpleStatuses (plus some
  * other random info).  For any buddy, or for any one of your accounts,
- * or for any person you're chatting with, you may know various
+ * or for any person with which you're chatting, you may know various
  * amounts of information.  This information is all contained in
  * one PurplePresence.  If one of your buddies is away and idle,
  * then the presence contains the PurpleStatus for their awayness,
  * and it contains their current idle time.  PurplePresences are
- * never saved to disk.  The information they contain is only relevent
+ * never saved to disk.  The information they contain is only relevant
  * for the current PurpleSession.
  */
 
+/**
+ * PurpleStatusType's are created by each PRPL.  They outline the
+ * available statuses of the protocol.  AIM, for example, supports
+ * an available state with an optional available message, an away
+ * state with a mandatory message, and an invisible state (which is
+ * technically "independent" of the other two, but we'll get into
+ * that later).  PurpleStatusTypes are very permanent.  They are
+ * hardcoded in each PRPL and will not change often.  And because
+ * they are hardcoded, they do not need to be saved to any XML file.
+ */
 typedef struct _PurpleStatusType      PurpleStatusType;
 typedef struct _PurpleStatusAttr      PurpleStatusAttr;
 typedef struct _PurplePresence        PurplePresence;
@@ -80,7 +90,7 @@ typedef struct _PurpleStatus          PurpleStatus;
 /**
  * A context for a presence.
  *
- * The context indicates what the presence applies to.
+ * The context indicates to what the presence applies.
  */
 typedef enum
 {
@@ -134,7 +144,7 @@ extern "C" {
 #endif
 
 /**************************************************************************/
-/** @name PurpleStatusPrimitive API                                         */
+/** @name PurpleStatusPrimitive API                                       */
 /**************************************************************************/
 /*@{*/
 
@@ -172,7 +182,7 @@ PurpleStatusPrimitive purple_primitive_get_type_from_id(const char *id);
 /*@}*/
 
 /**************************************************************************/
-/** @name PurpleStatusType API                                              */
+/** @name PurpleStatusType API                                            */
 /**************************************************************************/
 /*@{*/
 
@@ -199,8 +209,8 @@ PurpleStatusType *purple_status_type_new_full(PurpleStatusPrimitive primitive,
 										  gboolean independent);
 
 /**
- * Creates a new status type with some default values (not
- * savable and not independent).
+ * Creates a new status type with some default values (
+ * saveable and not independent).
  *
  * @param primitive     The primitive status type.
  * @param id            The ID of the status type, or @c NULL to use the id of
@@ -252,6 +262,7 @@ PurpleStatusType *purple_status_type_new_with_attrs(PurpleStatusPrimitive primit
  */
 void purple_status_type_destroy(PurpleStatusType *status_type);
 
+#if !(defined PURPLE_DISABLE_DEPRECATED) || (defined _PURPLE_STATUS_C_)
 /**
  * Sets a status type's primary attribute.
  *
@@ -261,10 +272,14 @@ void purple_status_type_destroy(PurpleStatusType *status_type);
  *
  * @param status_type The status type.
  * @param attr_id     The ID of the primary attribute.
+ *
+ * @deprecated This function isn't used and should be removed in 3.0.0.
  */
 void purple_status_type_set_primary_attr(PurpleStatusType *status_type,
 									   const char *attr_id);
+#endif
 
+#if !(defined PURPLE_DISABLE_DEPRECATED) || (defined _PURPLE_STATUS_C_)
 /**
  * Adds an attribute to a status type.
  *
@@ -272,10 +287,16 @@ void purple_status_type_set_primary_attr(PurpleStatusType *status_type,
  * @param id          The ID of the attribute.
  * @param name        The name presented to the user.
  * @param value       The value type of this attribute.
+ *
+ * @deprecated This function isn't needed and should be removed in 3.0.0.
+ *             Status type attributes should be set when the status type
+ *             is created, in the call to purple_status_type_new_with_attrs.
  */
 void purple_status_type_add_attr(PurpleStatusType *status_type, const char *id,
 							   const char *name, PurpleValue *value);
+#endif
 
+#if !(defined PURPLE_DISABLE_DEPRECATED) || (defined _PURPLE_STATUS_C_)
 /**
  * Adds multiple attributes to a status type.
  *
@@ -284,18 +305,29 @@ void purple_status_type_add_attr(PurpleStatusType *status_type, const char *id,
  * @param name        The description of the first attribute.
  * @param value       The value type of the first attribute attribute.
  * @param ...         Additional attribute information.
+ *
+ * @deprecated This function isn't needed and should be removed in 3.0.0.
+ *             Status type attributes should be set when the status type
+ *             is created, in the call to purple_status_type_new_with_attrs.
  */
 void purple_status_type_add_attrs(PurpleStatusType *status_type, const char *id,
 								const char *name, PurpleValue *value, ...) G_GNUC_NULL_TERMINATED;
+#endif
 
+#if !(defined PURPLE_DISABLE_DEPRECATED) || (defined _PURPLE_STATUS_C_)
 /**
  * Adds multiple attributes to a status type using a va_list.
  *
  * @param status_type The status type to add the attribute to.
  * @param args        The va_list of attributes.
+ *
+ * @deprecated This function isn't needed and should be removed in 3.0.0.
+ *             Status type attributes should be set when the status type
+ *             is created, in the call to purple_status_type_new_with_attrs.
  */
 void purple_status_type_add_attrs_vargs(PurpleStatusType *status_type,
 									  va_list args);
+#endif
 
 /**
  * Returns the primitive type of a status type.
@@ -378,14 +410,18 @@ gboolean purple_status_type_is_exclusive(const PurpleStatusType *status_type);
  */
 gboolean purple_status_type_is_available(const PurpleStatusType *status_type);
 
+#if !(defined PURPLE_DISABLE_DEPRECATED) || (defined _PURPLE_STATUS_C_)
 /**
  * Returns a status type's primary attribute ID.
  *
  * @param type The status type.
  *
  * @return The primary attribute's ID.
+ *
+ * @deprecated This function isn't used and should be removed in 3.0.0.
  */
 const char *purple_status_type_get_primary_attr(const PurpleStatusType *type);
+#endif
 
 /**
  * Returns the attribute with the specified ID.
@@ -537,35 +573,50 @@ void purple_status_set_active_with_attrs(PurpleStatus *status, gboolean active,
 void purple_status_set_active_with_attrs_list(PurpleStatus *status, gboolean active,
 											GList *attrs);
 
+#if !(defined PURPLE_DISABLE_DEPRECATED) || (defined _PURPLE_STATUS_C_)
 /**
  * Sets the boolean value of an attribute in a status with the specified ID.
  *
  * @param status The status.
  * @param id     The attribute ID.
  * @param value  The boolean value.
+ *
+ * @deprecated This function is only used by status.c and should be made
+ *             static in 3.0.0.
  */
 void purple_status_set_attr_boolean(PurpleStatus *status, const char *id,
 								  gboolean value);
+#endif
 
+#if !(defined PURPLE_DISABLE_DEPRECATED) || (defined _PURPLE_STATUS_C_)
 /**
  * Sets the integer value of an attribute in a status with the specified ID.
  *
  * @param status The status.
  * @param id     The attribute ID.
  * @param value  The integer value.
+ *
+ * @deprecated This function is only used by status.c and should be made
+ *             static in 3.0.0.
  */
 void purple_status_set_attr_int(PurpleStatus *status, const char *id,
 							  int value);
+#endif
 
+#if !(defined PURPLE_DISABLE_DEPRECATED) || (defined _PURPLE_STATUS_C_)
 /**
  * Sets the string value of an attribute in a status with the specified ID.
  *
  * @param status The status.
  * @param id     The attribute ID.
  * @param value  The string value.
+ *
+ * @deprecated This function is only used by status.c and should be made
+ *             static in 3.0.0.
  */
 void purple_status_set_attr_string(PurpleStatus *status, const char *id,
 								 const char *value);
+#endif
 
 /**
  * Returns the status's type.
@@ -773,22 +824,31 @@ PurplePresence *purple_presence_new_for_buddy(PurpleBuddy *buddy);
  */
 void purple_presence_destroy(PurplePresence *presence);
 
+#if !(defined PURPLE_DISABLE_DEPRECATED) || (defined _PURPLE_STATUS_C_)
 /**
  * Adds a status to a presence.
  *
  * @param presence The presence.
  * @param status   The status to add.
+ *
+ * @deprecated This function is only used by purple_presence_add_list,
+ *             and both should be removed in 3.0.0.
  */
 void purple_presence_add_status(PurplePresence *presence, PurpleStatus *status);
+#endif
 
+#if !(defined PURPLE_DISABLE_DEPRECATED) || (defined _PURPLE_STATUS_C_)
 /**
  * Adds a list of statuses to the presence.
  *
  * @param presence    The presence.
  * @param source_list The source list of statuses to add, which is not
  *                    modified or freed by this function.
+ *
+ * @deprecated This function isn't used and should be removed in 3.0.0.
  */
 void purple_presence_add_list(PurplePresence *presence, GList *source_list);
+#endif
 
 /**
  * Sets the active state of a status in a presence.
